/***********************************************************************************************************
 * Une classe représentant une Led connectée sur une pin arduino
 * 
 *  IUT GEII TOULON - Correction du TP n° 2
 *  M.GRIMALDI - Octobre 2020
 */

#include "Led.h"
#include <Arduino.h>    // ce fichier contient tous les prototypes des fonctions arduino (digitalWrite(), pinMode(), .etc) ainsi que toutes des constantes (HIGH, LOW, OUTPUT, .etc.)

//                    acesseurs de la classe setter/getter
int Led::get_pin()  {
  return pin;
}

bool Led::get_state()  {
  return state;
}

void Led::set_pin(int value) {
  if (validPin(value)){         // si le n° de pin est pas valide
	  pinMode(value, OUTPUT);
	  pin = value;
  }
  else pin=-1;          // si le n° de pin n'est pas valide, on met -1 dans pin
}

void Led::set_oneLogic(bool value) {
  oneLogic = value;
}

//                  constructeur par défaut: pin = -1 (non définie, state=false, oneLogic=true)
Led::Led(){
  pin=-1;           // -1 = pin invalide a priori
  state=0;          // la Led est éteinte
  oneLogic=true;    // allumage sur niveau logique "1" par défaut (branchée entre le 0V et une pin)
}

//                  constructeur:  pin =numéro de la pin Arduino, oneLogic= type de logique de commande (high ou low)
Led::Led(int pin, bool oneLogic){
  set_pin(pin);
  state=0;              // la Led est éteinte
  this->oneLogic=oneLogic;      // nota: on utilise ici this car le nom choisi pour l'argument (oneLogic) est
                                // le même que celui de l'attribut de la classe (oneLogic).
                                // this->oneLogic représente l'attribut et oneLogic (tout seul) représente
                                // l'argument de la fonction
}

//                  allume la led
void Led::lightOn() {
  if (pin==-1) return;      // pin invalide, on ne fait rien
  digitalWrite(pin, (oneLogic ? HIGH : LOW));
  state=true;               // la Led "se souvient" qu'elle est allumée
}

//                    éteind la led
 void Led::lightOff() {
  if (pin==-1) return;      // pin invalide, on ne fait rien
  digitalWrite(pin, (oneLogic ? LOW : HIGH));
  state=false;               // la Led "se souvient" qu'elle est éteinte
}

//                  Cette méthode inverse l'état de la led
 void Led::invert() {
     // si elle est allumée on l'éteint sinon on l'allume
     if (state==true)lightOff(); else lightOn();
}

//                  Cette méthode allume la led pendant ms millisecondes puis l'"teind
 void Led::flash(int ms) {
     if (state) return;     // déjé allumée, on ne fais rien!
     lightOn();
     delay(ms);
     lightOff();
}

//                  cette méthode vérifie si le n° de pin est valide (en fonction du type de carte: nano, uno, .etc.)
bool Led::validPin(int p) {
    return (p>=2 && p<=13);      // n° autorisés pour la carte nano (attention la pin 13 est déjà branchée sur la LED de la carte
}
