/***********************************************************************************************************
 * Une classe représentant un émetteur Morse utilisant Led connectée sur une pin arduino
 * 
 *  IUT GEII TOULON - Correction du TP n° 4
 *  M.GRIMALDI - Octobre 2020
 */
 
#include "Led.h"
#include "Morse.h"
#include <Arduino.h>

// ------------------------- l'alphabet morse --------------------------------------------------------------------------------------------------------------------------------------
const String codemorse[] = {".-.-.-","--..--","..--..", "-..-.",  ".-", "-...", "-.-.", "-..",".","..-.", "--.", "....", "..", ".---", "-.-", ".-..", "--", "-.", "---", ".--.", 
                            "--.-", ".-.", "...", "-", "..-", "...-", ".--", "-..-", "-.--", "--..",".----","..---","...--","....-",".....","-....","--...","---..","----.","-----"};
const String codealpha =".,?/ABCDEFGHIJKLMNOPQRSTUVWXYZ";


// -------------------- constructeur ---------------------------------
Morse::Morse(Led l, float sp){
  led = l;
  setSpeed(sp);
  led.lightOff();  
}

//------------------------------------------------------------------
void Morse::setSpeed(float v){
  if (v<0.5 || v>20) v=3.5;
  speedWPM = v;
  dotLength = 1200./speedWPM;
}

//------------------------------------------------------------------
float Morse::getSpeed(){
  return speedWPM;
}

//------------------------------------------------------------------
int Morse::getDotLength(){
  return dotLength;
}

//------------------------------------------------------------------
//  transmission d'un caractère
void Morse::send(char c){
    c = toupper(c);                         // conversion en majuscule
    int position = codealpha.indexOf(c);    // position de la lettre dans la liste
    if (position==-1) return;               // lettre non trouvée
    String codeChar = codemorse[position];  // le code morse de la lettre
    for (int i=0 ; i<codeChar.length() ; i++){
      if (codeChar[i]=='.')sendDot();
      else sendDash();
    }
  }

//------------------------------------------------------------------
// transmission d'une chaîne de caractère
void Morse::send(String s){
  for (int i=0 ; i<s.length() ; i++){
    if (s[i]==' ')delay(6*dotLength);     // 7 dots entre les mots
    else {
      send(s[i]);
      delay(2*dotLength);                 // 3 dots entre les lettres
    }
  }
  }

//------------------------------------------------------------------
// transmission d'un point
void Morse::sendDot(){ 
  led.flash(dotLength);
  delay(dotLength);
  }

//------------------------------------------------------------------
// transmission d'un trait
void Morse::sendDash(){
   led.flash(3 * dotLength);
  delay(dotLength);
  }
//------------------------------------------------------------------
